#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Color the chart background for up to three custom time frames.
    /// </summary>
    [Description("Color the chart background for up to three custom time frames.")]
    public class ColorTimeRegions : Indicator
    {
        #region Variables
        
            private int zn1HrSt 		= 0;
			private int zn1MinSt 		= 0;
        	private int zn1HrEn 		= 9;
			private int zn1MinEn 		= 15;
		
			private int zn2HrSt 		= 9;
			private int zn2MinSt 		= 15;
        	private int zn2HrEn 		= 16;
			private int zn2MinEn 		= 15;
		
			private int zn3HrSt 		= 16;
			private int zn3MinSt 		= 15;
        	private int zn3HrEn 		= 24;
			private int zn3MinEn 		= 0;
		
			private bool colorAll 		= false;
			
			private Color region1Color 	= Color.LightGray;
			private Color region2Color 	= Color.LightGreen;
			private Color region3Color 	= Color.LightGray;
		
			private bool alertBool			= true;
	
        #endregion

        protected override void Initialize()
        {
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
        }

        protected override void OnBarUpdate()
        {
			
		int CZn1St		= ((Zn1HrSt * 10000) + (Zn1MinSt * 100));
		int CZn1En	 	= ((Zn1HrEn * 10000) + (Zn1MinEn * 100));
			
		int CZn2St		= ((Zn2HrSt * 10000) + (Zn2MinSt * 100));
		int CZn2En	 	= ((Zn2HrEn * 10000) + (Zn2MinEn * 100));
			
		int CZn3St		= ((Zn3HrSt * 10000) + (Zn3MinSt * 100));
		int CZn3En	 	= ((Zn3HrEn * 10000) + (Zn3MinEn * 100));	
		
		if (ColorAll == false)
		{
			if (ToTime(Time[0]) >= CZn1St && ToTime(Time[0]) < CZn1En)
				BackColor = Region1Color;			
			
			if (ToTime(Time[0]) >= CZn2St && ToTime(Time[0]) < CZn2En)
				BackColor = Region2Color;			

			if (ToTime(Time[0]) >= CZn3St && ToTime(Time[0]) < CZn3En)
				BackColor = Region3Color;			

		}
		
		
		else if (ColorAll == true)
		{
			if (ToTime(Time[0]) >= CZn1St && ToTime(Time[0]) < CZn1En)
				BackColorAll = Region1Color;			
			
			if (ToTime(Time[0]) >= CZn2St && ToTime(Time[0]) < CZn2En)
				BackColorAll = Region2Color;			

		
			if (ToTime(Time[0]) >= CZn3St && ToTime(Time[0]) < CZn3En)
				BackColorAll = Region3Color;			
		}
		
		
		if (AlertBool == true)
			if (ToTime(Time[0]) == CZn1St)
				Alert("r1st",NinjaTrader.Cbi.Priority.Medium, "Beginning of Time Region #1","Alert2.wav",0,Region1Color,Color.Black);
			
			else if (ToTime(Time[0]) == CZn1En)
				Alert("r1en",NinjaTrader.Cbi.Priority.Medium, "End of Time Region #1","Alert2.wav",0,Region1Color,Color.Black);
		
			else if (ToTime(Time[0]) == CZn2St)
				Alert("r2st",NinjaTrader.Cbi.Priority.Medium, "Beginning of Time Region #2","Alert2.wav",0,Region2Color,Color.Black);
			
			else if (ToTime(Time[0]) == CZn2En)
				Alert("r2en",NinjaTrader.Cbi.Priority.Medium, "End of Time Region #2","Alert2.wav",0,Region2Color,Color.Black);
			
			else if (ToTime(Time[0]) == CZn3St)
				Alert("r3st",NinjaTrader.Cbi.Priority.Medium, "Beginning of Time Region #3","Alert2.wav",0,Region3Color,Color.Black);
			
			else if (ToTime(Time[0]) == CZn3En)
				Alert("r3en",NinjaTrader.Cbi.Priority.Medium, "End of Time Region #3","Alert2.wav",0,Region3Color,Color.Black);
			
        }

        #region Properties

        [Description("Hour for 1st time region to begin, 24 hour clock.")]
        [Category("1st Time Region")]
		[Gui.Design.DisplayNameAttribute("Begin Hour")]
        public int Zn1HrSt
        {
            get { return zn1HrSt; }
            set { zn1HrSt = Math.Max(0, value); }
        }
		
		
		[Description("Minute for 1st time region to begin")]
        [Category("1st Time Region")]
		[Gui.Design.DisplayNameAttribute("Begin Minute")]
        public int Zn1MinSt
        {
            get { return zn1MinSt; }
            set { zn1MinSt = Math.Max(0, value); }
        }
		
		
		[Description("Hour for 1st time region to end, 24 hour clock.")]
        [Category("1st Time Region")]
		[Gui.Design.DisplayNameAttribute("End Hour")]
        public int Zn1HrEn
        {
            get { return zn1HrEn; }
            set { zn1HrEn = Math.Max(0, value); }
        }
		
		
		[Description("Minute for 1st time region to end")]
        [Category("1st Time Region")]
		[Gui.Design.DisplayNameAttribute("End Minute")]
        public int Zn1MinEn
        {
            get { return zn1MinEn; }
            set { zn1MinEn = Math.Max(0, value); }
        }
		
		
		[Description("Hour for 2nd time region to begin, 24 hour clock.")]
        [Category("2nd Time Region")]
		[Gui.Design.DisplayNameAttribute("Begin Hour")]
        public int Zn2HrSt
        {
            get { return zn2HrSt; }
            set { zn2HrSt = Math.Max(0, value); }
        }
		
		
		[Description("Minute for 2nd time region to begin")]
        [Category("2nd Time Region")]
		[Gui.Design.DisplayNameAttribute("Begin Minute")]
        public int Zn2MinSt
        {
            get { return zn2MinSt; }
            set { zn2MinSt = Math.Max(0, value); }
        }
		
		
		[Description("Hour for 2nd time region to end, 24 hour clock.")]
        [Category("2nd Time Region")]
		[Gui.Design.DisplayNameAttribute("End Hour")]
        public int Zn2HrEn
        {
            get { return zn2HrEn; }
            set { zn2HrEn = Math.Max(0, value); }
        }
		
		
		[Description("Minute for 2nd time region to end")]
        [Category("2nd Time Region")]
		[Gui.Design.DisplayNameAttribute("End Minute")]
        public int Zn2MinEn
        {
            get { return zn2MinEn; }
            set { zn2MinEn = Math.Max(0, value); }
        }
	
		
		[Description("Hour for 3rd time region to begin, 24 hour clock.")]
        [Category("3rd Time Region")]
		[Gui.Design.DisplayNameAttribute("Begin Hour")]
        public int Zn3HrSt
        {
            get { return zn3HrSt; }
            set { zn3HrSt = Math.Max(0, value); }
        }
		
		
		[Description("Minute for 3rd time region to begin")]
        [Category("3rd Time Region")]
		[Gui.Design.DisplayNameAttribute("Begin Minute")]
        public int Zn3MinSt
        {
            get { return zn3MinSt; }
            set { zn3MinSt = Math.Max(0, value); }
        }
		
		
		[Description("Hour for 3rd time region to end, 24 hour clock.")]
        [Category("3rd Time Region")]
		[Gui.Design.DisplayNameAttribute("End Hour")]
        public int Zn3HrEn
        {
            get { return zn3HrEn; }
            set { zn3HrEn = Math.Max(0, value); }
        }
		
		
		[Description("Minute for 3rd time region to end")]
        [Category("3rd Time Region")]
		[Gui.Design.DisplayNameAttribute("End Minute")]
        public int Zn3MinEn
        {
            get { return zn3MinEn; }
            set { zn3MinEn = Math.Max(0, value); }
        }

		
		[Description("Colors across all panels or just main price display panel")]
        [Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Color all?")]
        public bool ColorAll
        {
            get { return colorAll; }
            set { colorAll = value; }
        }
		
		
		[XmlIgnore()]
        [Description("Color for 1st region")]
        [Category("1st Time Region")]
		[Gui.Design.DisplayNameAttribute("Region Color")]
        public Color Region1Color
        {
            get { return region1Color; }
            set { region1Color = value; }
        }
		
		[Browsable(false)]
		public string Region1ColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(region1Color); }
			set { region1Color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
        [Description("Color for 2nd region")]
        [Category("2nd Time Region")]
		[Gui.Design.DisplayNameAttribute("Region Color")]
        public Color Region2Color
        {
            get { return region2Color; }
            set { region2Color = value; }
        }
		
		[Browsable(false)]
		public string Region2ColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(region2Color); }
			set { region2Color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
        [Description("Color for 3rd region")]
        [Category("3rd Time Region")]
		[Gui.Design.DisplayNameAttribute("Region Color")]
        public Color Region3Color
        {
            get { return region3Color; }
            set { region3Color = value; }
        }
		
		[Browsable(false)]
		public string Region3ColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(region3Color); }
			set { region3Color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[Description("Sound and display an alert with Alerts window open ")]
        [Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Alert on Begin/End?")]
        public bool AlertBool
        {
            get { return alertBool; }
            set { alertBool = value; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ColorTimeRegions[] cacheColorTimeRegions = null;

        private static ColorTimeRegions checkColorTimeRegions = new ColorTimeRegions();

        /// <summary>
        /// Color the chart background for up to three custom time frames.
        /// </summary>
        /// <returns></returns>
        public ColorTimeRegions ColorTimeRegions(bool alertBool, bool colorAll)
        {
            return ColorTimeRegions(Input, alertBool, colorAll);
        }

        /// <summary>
        /// Color the chart background for up to three custom time frames.
        /// </summary>
        /// <returns></returns>
        public ColorTimeRegions ColorTimeRegions(Data.IDataSeries input, bool alertBool, bool colorAll)
        {
            if (cacheColorTimeRegions != null)
                for (int idx = 0; idx < cacheColorTimeRegions.Length; idx++)
                    if (cacheColorTimeRegions[idx].AlertBool == alertBool && cacheColorTimeRegions[idx].ColorAll == colorAll && cacheColorTimeRegions[idx].EqualsInput(input))
                        return cacheColorTimeRegions[idx];

            lock (checkColorTimeRegions)
            {
                checkColorTimeRegions.AlertBool = alertBool;
                alertBool = checkColorTimeRegions.AlertBool;
                checkColorTimeRegions.ColorAll = colorAll;
                colorAll = checkColorTimeRegions.ColorAll;

                if (cacheColorTimeRegions != null)
                    for (int idx = 0; idx < cacheColorTimeRegions.Length; idx++)
                        if (cacheColorTimeRegions[idx].AlertBool == alertBool && cacheColorTimeRegions[idx].ColorAll == colorAll && cacheColorTimeRegions[idx].EqualsInput(input))
                            return cacheColorTimeRegions[idx];

                ColorTimeRegions indicator = new ColorTimeRegions();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.AlertBool = alertBool;
                indicator.ColorAll = colorAll;
                Indicators.Add(indicator);
                indicator.SetUp();

                ColorTimeRegions[] tmp = new ColorTimeRegions[cacheColorTimeRegions == null ? 1 : cacheColorTimeRegions.Length + 1];
                if (cacheColorTimeRegions != null)
                    cacheColorTimeRegions.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheColorTimeRegions = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Color the chart background for up to three custom time frames.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ColorTimeRegions ColorTimeRegions(bool alertBool, bool colorAll)
        {
            return _indicator.ColorTimeRegions(Input, alertBool, colorAll);
        }

        /// <summary>
        /// Color the chart background for up to three custom time frames.
        /// </summary>
        /// <returns></returns>
        public Indicator.ColorTimeRegions ColorTimeRegions(Data.IDataSeries input, bool alertBool, bool colorAll)
        {
            return _indicator.ColorTimeRegions(input, alertBool, colorAll);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Color the chart background for up to three custom time frames.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ColorTimeRegions ColorTimeRegions(bool alertBool, bool colorAll)
        {
            return _indicator.ColorTimeRegions(Input, alertBool, colorAll);
        }

        /// <summary>
        /// Color the chart background for up to three custom time frames.
        /// </summary>
        /// <returns></returns>
        public Indicator.ColorTimeRegions ColorTimeRegions(Data.IDataSeries input, bool alertBool, bool colorAll)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ColorTimeRegions(input, alertBool, colorAll);
        }
    }
}
#endregion
