#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Identify flags
    /// </summary>
    [Description("Identify flags")]
    public class FlagIdent : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
        private double distanceSMA = 0.00024; // Default setting for DistanceSMA
		private double belowSMA = 0.00000;
		private double aboveSMA = 0.00000;
		private double entry = 0.00000;
		private double takeProfit = 0.00000;
		private double stop = 0.00000;
		private bool validSUB = false;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            // Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "Plot0"));
            Overlay				= true;
			CalculateOnBarClose = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
            // Plot0.Set(Close[0]);
			            // Initialization at each barupdate
			belowSMA = SMAMedian(8)[0] - distanceSMA;
			aboveSMA = SMAMedian(8)[0] + distanceSMA;
			validSUB = false;
			
			// SUB should be touching SMA or within 2.4 pips of it.
			if (High[0] >= SMAMedian(8)[0]
				&& Low[0] <= SMAMedian(8)[0])
				validSUB = true;
			else if (Low[0] <= aboveSMA)
				validSUB = true;
			else if (High[0] >= belowSMA)
				validSUB = true;
			
			// Condition set 1
            if (CCI(8)[0] >= 50
                && Close[0] > SMAMedian(8)[0]
                && CCI(8)[1] <= 50
				&& validSUB == true)
            {
//                if (High[0] - Low[0] <= 0.0006)
//					stop = Low[0] - (0.0009 - (High[0] - Low[0]) - 0.0001);
//				else
//					stop = Low[0] - 0.0001;
				
				entry = High[0] + 0.0001;
				stop = entry - 0.0015;
				takeProfit = entry + 0.0015;
				DrawLine("stop", false, 1, stop, 0, stop, Color.White, DashStyle.Dot, 6);
				DrawLine("entry", false, 1, entry, 0, entry, Color.White, DashStyle.Dot, 6);
				DrawLine("takeProfit", false, 1, takeProfit, 0, takeProfit, Color.White, DashStyle.Dot, 6);
//				int rounded = (int)Math.Round((High[0] - Low[0]) * 10000);
//				if (rounded < 9)
//					rounded = 9;
//				DrawText("SUB", rounded.ToString(), 0, High[0] + 0.0005, Color.White);
				
				BarColor = Color.Yellow;
                PlaySound(@"D:\Program Files\NinjaTrader 7\sounds\Alert2.wav");
            }

            // Condition set 2
            if (CCI(8)[0] <= -50
                && Close[0] < SMAMedian(8)[0]
                && CCI(8)[1] >= -50
				&& validSUB == true)
            {
//                if (High[0] - Low[0] <= 0.0006)
//					stop = High[0] + (0.0009 - (High[0] - Low[0]) - 0.0001);
//				else
//					stop = High[0] + 0.0001;
				
				entry = Low[0] - 0.0001;
				stop = entry + 0.0015;
				takeProfit = entry - 0.0015;
				DrawLine("stop", false, 1, stop, 0, stop, Color.White, DashStyle.Dot, 6);
				DrawLine("entry", false, 1, entry, 0, entry, Color.White, DashStyle.Dot, 6);
				DrawLine("takeProfit", false, 1, takeProfit, 0, takeProfit, Color.White, DashStyle.Dot, 6);
//				int rounded = (int)Math.Round((High[0] - Low[0]) * 10000);
//				if (rounded < 9)
//					rounded = 9;
//				DrawText("SUB", rounded.ToString(), 0, High[0] + 0.0005, Color.White);
				
				BarColor = Color.Blue;
                PlaySound(@"D:\Program Files\NinjaTrader 7\sounds\Alert2.wav");
            }

        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private FlagIdent[] cacheFlagIdent = null;

        private static FlagIdent checkFlagIdent = new FlagIdent();

        /// <summary>
        /// Identify flags
        /// </summary>
        /// <returns></returns>
        public FlagIdent FlagIdent()
        {
            return FlagIdent(Input);
        }

        /// <summary>
        /// Identify flags
        /// </summary>
        /// <returns></returns>
        public FlagIdent FlagIdent(Data.IDataSeries input)
        {
            if (cacheFlagIdent != null)
                for (int idx = 0; idx < cacheFlagIdent.Length; idx++)
                    if (cacheFlagIdent[idx].EqualsInput(input))
                        return cacheFlagIdent[idx];

            lock (checkFlagIdent)
            {
                if (cacheFlagIdent != null)
                    for (int idx = 0; idx < cacheFlagIdent.Length; idx++)
                        if (cacheFlagIdent[idx].EqualsInput(input))
                            return cacheFlagIdent[idx];

                FlagIdent indicator = new FlagIdent();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                FlagIdent[] tmp = new FlagIdent[cacheFlagIdent == null ? 1 : cacheFlagIdent.Length + 1];
                if (cacheFlagIdent != null)
                    cacheFlagIdent.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheFlagIdent = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Identify flags
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FlagIdent FlagIdent()
        {
            return _indicator.FlagIdent(Input);
        }

        /// <summary>
        /// Identify flags
        /// </summary>
        /// <returns></returns>
        public Indicator.FlagIdent FlagIdent(Data.IDataSeries input)
        {
            return _indicator.FlagIdent(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Identify flags
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FlagIdent FlagIdent()
        {
            return _indicator.FlagIdent(Input);
        }

        /// <summary>
        /// Identify flags
        /// </summary>
        /// <returns></returns>
        public Indicator.FlagIdent FlagIdent(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.FlagIdent(input);
        }
    }
}
#endregion
