// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The SMAMedian (Simple Moving Average) is an indicator that shows the average value of a security's price over a period of time.
	/// </summary>
	[Description("The SMAMedian (Simple Moving Average) is an indicator that shows the average value of a security's price over a period of time.")]
	public class SMAMedian : Indicator
	{
		#region Variables
		private int		period	= 14;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Orange, "SMAMedian"));

			Overlay = true;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick).
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar == 0)
				Value.Set(Median[0]);
			else
			{
				double last = Value[1] * Math.Min(CurrentBar, Period);

				if (CurrentBar >= Period)
					Value.Set((last + Median[0] - Median[Period]) / Math.Min(CurrentBar, Period));
				else
					Value.Set((last + Median[0]) / (Math.Min(CurrentBar, Period) + 1));
			}
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[GridCategory("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
        #endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SMAMedian[] cacheSMAMedian = null;

        private static SMAMedian checkSMAMedian = new SMAMedian();

        /// <summary>
        /// The SMAMedian (Simple Moving Average) is an indicator that shows the average value of a security's price over a period of time.
        /// </summary>
        /// <returns></returns>
        public SMAMedian SMAMedian(int period)
        {
            return SMAMedian(Input, period);
        }

        /// <summary>
        /// The SMAMedian (Simple Moving Average) is an indicator that shows the average value of a security's price over a period of time.
        /// </summary>
        /// <returns></returns>
        public SMAMedian SMAMedian(Data.IDataSeries input, int period)
        {
            if (cacheSMAMedian != null)
                for (int idx = 0; idx < cacheSMAMedian.Length; idx++)
                    if (cacheSMAMedian[idx].Period == period && cacheSMAMedian[idx].EqualsInput(input))
                        return cacheSMAMedian[idx];

            lock (checkSMAMedian)
            {
                checkSMAMedian.Period = period;
                period = checkSMAMedian.Period;

                if (cacheSMAMedian != null)
                    for (int idx = 0; idx < cacheSMAMedian.Length; idx++)
                        if (cacheSMAMedian[idx].Period == period && cacheSMAMedian[idx].EqualsInput(input))
                            return cacheSMAMedian[idx];

                SMAMedian indicator = new SMAMedian();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Period = period;
                Indicators.Add(indicator);
                indicator.SetUp();

                SMAMedian[] tmp = new SMAMedian[cacheSMAMedian == null ? 1 : cacheSMAMedian.Length + 1];
                if (cacheSMAMedian != null)
                    cacheSMAMedian.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSMAMedian = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The SMAMedian (Simple Moving Average) is an indicator that shows the average value of a security's price over a period of time.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SMAMedian SMAMedian(int period)
        {
            return _indicator.SMAMedian(Input, period);
        }

        /// <summary>
        /// The SMAMedian (Simple Moving Average) is an indicator that shows the average value of a security's price over a period of time.
        /// </summary>
        /// <returns></returns>
        public Indicator.SMAMedian SMAMedian(Data.IDataSeries input, int period)
        {
            return _indicator.SMAMedian(input, period);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The SMAMedian (Simple Moving Average) is an indicator that shows the average value of a security's price over a period of time.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SMAMedian SMAMedian(int period)
        {
            return _indicator.SMAMedian(Input, period);
        }

        /// <summary>
        /// The SMAMedian (Simple Moving Average) is an indicator that shows the average value of a security's price over a period of time.
        /// </summary>
        /// <returns></returns>
        public Indicator.SMAMedian SMAMedian(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SMAMedian(input, period);
        }
    }
}
#endregion
